// SwatchMaker.js
// An InDesign CS JavaScript
// 7 JUN 2006
// Copyright (C) 2006  Scott Zanelli. Lonelytree Software. (www.lonelytreesw.com)

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

// Ensure the dialog box will be shown.
	if (app.version == 3) {
		app.userInteractionLevel = UserInteractionLevels.interactWithAll;
	} 
	else {
		app.scriptPreferences.userInteractionLevel = UserInteractionLevels.interactWithAll;
	}

// See if a document is open. If not, exit
if (app.documents.length == 0)
	goodBye("Please open a document and try again.");

// Default Preferences
var cynStart       = 0;
var magStart       = 0;
var yelStart       = 0;
var blkStart       = 0;
var cynIncrement   = 1;
var magIncrement   = 1;
var yelIncrement   = 1;
var blkIncrement   = 1;
var steps          = 4;
var doCombinations = 0;

// Needed variables to start
var boxOffset;
var squareSize;
var theDoc   = app.activeDocument;
var units    = theDoc.viewPreferences.verticalMeasurementUnits;
var pgHeight = theDoc.documentPreferences.pageHeight;
var pgWidth  = theDoc.documentPreferences.pageWidth;
switch (units){// Use correct values depending on units
	case MeasurementUnits.inches: 
	case MeasurementUnits.inchesDecimal:
		boxOffset  = 0.25;
		squareSize = 0.5;
	break;
	case MeasurementUnits.points:
		boxOffset  = 18.0;
		squareSize = 36.0;
	break;
	case MeasurementUnits.picas:
		boxOffset  = 1.5;
		squareSize = 3.0;
	break;
	case MeasurementUnits.millimeters:
		boxOffset  = 6.35;
		squareSize = 12.7;
	break;
	case MeasurementUnits.centimeters:
		boxOffset  = 0.635;
		squareSize = 1.27;
	break;
	case MeasurementUnits.ciceros:
		boxOffset  = 1.4075;
		squareSize = 2.815;
	break;
}

// Check if preferences file exists and load it, if not save one
var prefsFile = File((Folder(app.activeScript)).parent + "/SwatchMakerPrefs.txt");
if (!prefsFile.exists)
  savePrefs();
else
	readPrefs();

do{
	var theDialog = createDialog();
	if(!(theDialog.show() ) )
		goodBye("User canceled.");
	
	var startT = new Date();
			
	// Read the dialog info
	var NEWcynStart       = Number(NEWcynStart.editContents);	
	var NEWmagStart       = Number(NEWmagStart.editContents);	
	var NEWyelStart       = Number(NEWyelStart.editContents);	
	var NEWblkStart       = Number(NEWblkStart.editContents);	
	var NEWcynIncrement   = Number(NEWcynIncrement.editContents);	
	var NEWmagIncrement   = Number(NEWmagIncrement.editContents);	
	var NEWyelIncrement   = Number(NEWyelIncrement.editContents);	
	var NEWblkIncrement   = Number(NEWblkIncrement.editContents);	
	var NEWsteps          = Number(NEWsteps.editContents);
	var NEWsquareSize     = Number(NEWsquareSize.editContents);	
	var NEWdoCombinations = (NEWdoCombinations.checkedState)?1:0
	var boxesX            = Math.floor((pgWidth  - 2*boxOffset)/(NEWsquareSize + boxOffset));
	var boxesY            = Math.floor((pgHeight - 2*boxOffset)/(NEWsquareSize + boxOffset));

	// Check for erroneous input
	if (isNaN(NEWcynStart)    ||isNaN(NEWmagStart)    ||isNaN(NEWyelStart)    ||isNaN(NEWblkStart)||
	    isNaN(NEWcynIncrement)||isNaN(NEWmagIncrement)||isNaN(NEWyelIncrement)||isNaN(NEWblkIncrement)||
	    isNaN(NEWsteps)       ||isNaN(NEWsquareSize) ){
    alert("Non-numeric entered. Please try again.");
    theDialog.destroy();
  }
  else if (NEWcynStart + (NEWsteps-1)*NEWcynIncrement > 100 || NEWmagStart + (NEWsteps-1)*NEWmagIncrement > 100 ||
           NEWyelStart + (NEWsteps-1)*NEWyelIncrement > 100 || NEWblkStart + (NEWsteps-1)*NEWblkIncrement > 100 ){
	  alert("A start/increment/step combination results in a final value over 100%. Please try again.");
	  theDialog.destroy();
  }
  else if (NEWcynStart < 0 || NEWmagStart < 0 || NEWyelStart < 0 || NEWblkStart < 0){
	  alert("Start values must be positive. Please try again.");
	  theDialog.destroy();
  }
  else if (NEWsteps < 2 || NEWsteps > 10){
	  alert("Step value must be between 2 and 10. Please try again.");
	  theDialog.destroy();
  }
  else if(NEWsquareSize < 2*boxOffset){
	  alert("Size must be " + (2*boxOffset) + " or greater. Please try again.");
	  theDialog.destroy();
  }
  else if(boxesX == 0 || boxesY ==0){
	  alert("Swatch size too large for page size. Either use a larger page or smaller Swatch Size. Please try again.");
	  theDialog.destroy();
  }
	else
		break;
}while(true);

//Remove the dialog from memory
theDialog.destroy();

// Entered Info is OK, use it
cynStart       = NEWcynStart;
magStart       = NEWmagStart;
yelStart       = NEWyelStart;
blkStart       = NEWblkStart;
cynIncrement   = NEWcynIncrement;
magIncrement   = NEWmagIncrement;
yelIncrement   = NEWyelIncrement;
blkIncrement   = NEWblkIncrement;
steps          = NEWsteps;
doCombinations = NEWdoCombinations;
squareSize     = NEWsquareSize;
savePrefs();

var numColorsChanging = 0;

// Determine swatch types
var swatchType = 0;
if (cynIncrement != 0){
	swatchType |= 1;
	numColorsChanging++;
}
if (magIncrement != 0){
	swatchType |= 2;
	numColorsChanging++;
}
if (yelIncrement != 0){
	swatchType |= 4;
	numColorsChanging++;
}
if (blkIncrement != 0){
	swatchType |= 8;
	numColorsChanging++;
}
	
// Box calculations
if (boxesX != 1)
	var boxWidthAdjust  = (pgWidth  - 2*boxOffset - squareSize)/(boxesX-1);
else // Only one box in this direction
	var boxWidthAdjust  = 0;

if (boxesY != 1)
	var boxHeightAdjust = (pgHeight - 2.9152*boxOffset - squareSize)/(boxesY-1);
else // Only one box in this direction
	var boxHeightAdjust = 0;
var xPosition = 0;
var yPosition = 0;
var page      = 0;

// Add the required number of pages.
try{
if(doCombinations)
	theDoc.documentPreferences.pagesPerDocument = Math.ceil(Math.pow(steps, numColorsChanging)/(boxesX*boxesY) );
else
	theDoc.documentPreferences.pagesPerDocument = Math.ceil(((steps-1)*numColorsChanging +1)/(boxesX*boxesY) );
}
catch(e){/*An error indicates only one page was needed*/;}

// Adjust for zero based arrays
boxesX--;
boxesY--;

// Set the text to fit in smallest box size that ever gets used
with(theDoc.textDefaults){
	horizontalScale = 58;
	pointSize = 7;
	justification = Justification.centerAlign;
}

// Set the origin to be per page and to be 0,0
theDoc.viewPreferences.rulerOrigin = RulerOrigin.pageOrigin;
theDoc.zeroPoint = [0,0];

// Swatch percentages
var cVal = 0;
var mVal = 0;
var yVal = 0;
var kVal = 0;
if (doCombinations){// Make iterative combinations (#colors = steps^(#colors incrementing))
	switch (swatchType){
		case 1:// C
			for(c = 0; c < steps; c++){
				cVal = cynStart + c * cynIncrement;
				addColor(cVal, magStart, yelStart, blkStart);
			}
		break;
		case 2:// M
			for(m = 0; m < steps; m++){
				mVal = magStart + m * magIncrement;
				addColor(cynStart, mVal, yelStart, blkStart);
			}
		break;
		case 3:// CM
			for(c = 0; c < steps; c++){
				cVal = cynStart + c * cynIncrement;
				for(m = 0; m < steps; m++){
					mVal = magStart + m * magIncrement;
					addColor(cVal, mVal, yelStart, blkStart);
				}
			}
		break;
		case 4:// Y
			for(y = 0; y< steps; y++){
				yVal = yelStart + y * yelIncrement;
				addColor(cynStart, magStart, yVal, blkStart);
			}
		break;
		case 5:// CY
			for(c = 0; c < steps; c++){
				cVal = cynStart + c * cynIncrement;
				for(y = 0; y< steps; y++){
					yVal = yelStart + y * yelIncrement;
					addColor(cVal, magStart, yVal, blkStart);
				}
			}
		break;
		case 6:// MY
			for(m = 0; m < steps; m++){
				mVal = magStart + m * magIncrement;
				for(y = 0; y< steps; y++){
					yVal = yelStart + y * yelIncrement;
						addColor(cynStart, mVal, yVal, blkStart);
				}
			}
		break;
		case 7:// CMY
			for(c = 0; c < steps; c++){
				cVal = cynStart + c * cynIncrement;
				for(m = 0; m < steps; m++){
					mVal = magStart + m * magIncrement;
					for(y = 0; y< steps; y++){
						yVal = yelStart + y * yelIncrement;
						addColor(cVal, mVal, yVal, blkStart);
					}
				}
			}
		break;
		case 8:// K
			for(k = 0; k < steps; k++){
				kVal = blkStart + k * blkIncrement;
				addColor(cynStart, magStart, yelStart, kVal);
			}
		break;
		case 9:// CK
			for(c = 0; c < steps; c++){
				cVal = cynStart + c * cynIncrement;
				for(k = 0; k < steps; k++){
					kVal = blkStart + k * blkIncrement;
					addColor(cVal, magStart, yelStart, kVal);
				}
			}
		break;
		case 10:// MK
			for(m = 0; m < steps; m++){
				mVal = magStart + m * magIncrement;
				for(k = 0; k < steps; k++){
					kVal = blkStart + k * blkIncrement;
					addColor(cynStart, mVal, yelStart, kVal);
				}
			}
		break;
		case 11:// CMK
			for(c = 0; c < steps; c++){
				cVal = cynStart + c * cynIncrement;
				for(m = 0; m < steps; m++){
					mVal = magStart + m * magIncrement;
					for(k = 0; k < steps; k++){
						kVal = blkStart + k * blkIncrement;
						addColor(cVal, mVal, yelStart, kVal);
					}
				}
			}
		break;
		case 12:// YK
			for(y = 0; y< steps; y++){
				yVal = yelStart + y * yelIncrement;
				for(k = 0; k < steps; k++){
					kVal = blkStart + k * blkIncrement;
					addColor(cynStart, magStart, yVal, kVal);
				}
			}
		break;
		case 13:// CYK
			for(c = 0; c < steps; c++){
				cVal = cynStart + c * cynIncrement;
				for(y = 0; y< steps; y++){
					yVal = yelStart + y * yelIncrement;
					for(k = 0; k < steps; k++){
						kVal = blkStart + k * blkIncrement;
						addColor(cVal, magStart, yVal, kVal);
					}
				}
			}
		break;
		case 14:// MYK
			for(m = 0; m < steps; m++){
				mVal = magStart + m * magIncrement;
				for(y = 0; y< steps; y++){
					yVal = yelStart + y * yelIncrement;
					for(k = 0; k < steps; k++){
						kVal = blkStart + k * blkIncrement;
						addColor(cynStart, mVal, yVal, kVal);
					}
				}
			}
		break;
		case 15:// CMYK
			for(c = 0; c < steps; c++){
				cVal = cynStart + c * cynIncrement;
				for(m = 0; m < steps; m++){
					mVal = magStart + m * magIncrement;
					for(y = 0; y< steps; y++){
						yVal = yelStart + y * yelIncrement;
						for(k = 0; k < steps; k++){
							kVal = blkStart + k * blkIncrement;
							addColor(cVal, mVal, yVal, kVal);
						}
					}
				}
			}
		break;
	}
}
else {// Do simple "combinations"
	addColor(cynStart, magStart, yelStart, blkStart);
	for(i = 1;i < steps; i++){
		cynVal = cynStart + (cynIncrement * i);
		magVal = magStart + (magIncrement * i);
		yelVal = yelStart + (yelIncrement * i);
		blkVal = blkStart + (blkIncrement * i);
		
		if (cynIncrement > 0){
			addColor(cynVal, magStart, yelStart, blkStart);
			if (magIncrement > 0)
				addColor(cynVal, magVal, yelStart, blkStart);
			if (yelIncrement > 0)
				addColor(cynVal, magStart, yelVal, blkStart);
			if (blkIncrement > 0)
				addColor(cynVal, magStart, yelStart, blkVal);
		}		
		if (magIncrement > 0){
			addColor(cynStart, magVal, yelStart, blkStart);
			if (yelIncrement > 0)
				addColor(cynStart, magVal, yelVal, blkStart);
			if (blkIncrement > 0)
				addColor(cynStart, magVal, yelStart, blkVal);
		}
		if (yelIncrement > 0){
			addColor(cynStart, magStart, yelVal, blkStart);
			if (blkIncrement > 0)
				addColor(cynStart, magStart, yelVal, blkVal);
		}
		if (blkIncrement > 0)
			addColor(cynStart, magStart, yelStart, blkVal);
		if (cynIncrement > 0 && magIncrement > 0){
			if (yelIncrement > 0)
				addColor(cynVal, magVal, yelVal, blkStart);
			if (blkIncrement > 0)
				addColor(cynVal, magVal, yelStart, blkVal);
		}
		if (cynIncrement > 0 && yelIncrement > 0 && blkIncrement > 0)
			addColor(cynVal, magStart, yelVal, blkVal);
		
		if (magIncrement > 0 && yelIncrement > 0 && blkIncrement > 0)
			addColor(cynStart, magVal, yelVal, blkVal);
		
		if (cynIncrement > 0 && magIncrement > 0 && yelIncrement > 0 && blkIncrement > 0)
			addColor(cynVal, magVal, yelVal, blkVal);
	}
}

// Clean up and say toodles
var endT = new Date();
goodBye("Finished creating swatches: " + ((endT.getTime() - startT.getTime())/1000) + " sec." );

/*
 * Create the dialog
 */
function createDialog(){
	var tempDialog = app.dialogs.add({name:"Swatch Maker 2.0", canCancel:true} );
	with (tempDialog){
		with (dialogColumns.add() ){
			with (dialogRows.add() )
				staticTexts.add({staticLabel:"Start Color (%)"} );
			with(borderPanels.add() ){
				with(dialogColumns.add() ){
					with (dialogRows.add() ){
						staticTexts.add({staticLabel:"C"} );
						NEWcynStart = textEditboxes.add({editContents:""+cynStart, minWidth: 30} );
						staticTexts.add({staticLabel:"M"} );
						NEWmagStart = textEditboxes.add({editContents:""+magStart, minWidth: 30} );
						staticTexts.add({staticLabel:"Y"} );
						NEWyelStart = textEditboxes.add({editContents:""+yelStart, minWidth: 30} );
						staticTexts.add({staticLabel:"K"} );
						NEWblkStart = textEditboxes.add({editContents:""+blkStart, minWidth: 30});
					}
				}
			}
			with (dialogRows.add() )
				staticTexts.add({staticLabel:"Increment (%)"} );
			with(borderPanels.add() ){
				with(dialogColumns.add() ){
					with (dialogRows.add() ){
						staticTexts.add({staticLabel:"C"} );
						NEWcynIncrement = textEditboxes.add({editContents:""+cynIncrement, minWidth: 30} );
						staticTexts.add({staticLabel:"M"} );
						NEWmagIncrement = textEditboxes.add({editContents:""+magIncrement, minWidth: 30} );
						staticTexts.add({staticLabel:"Y"} );
						NEWyelIncrement = textEditboxes.add({editContents:""+yelIncrement, minWidth: 30} );
						staticTexts.add({staticLabel:"K"} );
						NEWblkIncrement = textEditboxes.add({editContents:""+blkIncrement, minWidth: 30});
					}
					with(dialogRows.add() )
						staticTexts.add({staticLabel:""} );
					with(dialogRows.add() ){
						staticTexts.add({staticLabel:"Steps (2-10)"} );
						NEWsteps = textEditboxes.add({editContents:""+steps, minWidth: 30} );
						staticTexts.add({staticLabel:"       "} );
						NEWdoCombinations = checkboxControls.add({staticLabel:"Combinations", checkedState:doCombinations} )			
					}
				}
			}
			with (dialogRows.add() ){
				staticTexts.add({staticLabel:"Swatch Size"} );
				NEWsquareSize = textEditboxes.add({editContents:""+squareSize, minWidth: 30} );
			}
		}
	}
	return tempDialog;
}

/* 
 * Save the preferences to a text file
 */
function savePrefs(){
	var newPrefs = 
	cynStart       + "\n" +
	magStart       + "\n" +
	yelStart       + "\n" +
	blkStart       + "\n" +
	cynIncrement   + "\n" + 
	magIncrement   + "\n" + 
	yelIncrement   + "\n" +
	blkIncrement   + "\n" + 
	steps          + "\n" +
	doCombinations + "\n" +
	squareSize;
	prefsFile.open("w");
	prefsFile.write(newPrefs);
	prefsFile.close();
}

/* 
 * Read the preferences from a text file
 */
 function readPrefs(){
	prefsFile.open("r");
	cynStart       = Number(prefsFile.readln() );
	magStart       = Number(prefsFile.readln() ); 
	yelStart       = Number(prefsFile.readln() ); 
	blkStart       = Number(prefsFile.readln() ); 
	cynIncrement   = Number(prefsFile.readln() ); 
	magIncrement   = Number(prefsFile.readln() );
	yelIncrement   = Number(prefsFile.readln() );
	blkIncrement   = Number(prefsFile.readln() ); 
	steps          = Number(prefsFile.readln() ); 
	doCombinations = Number(prefsFile.readln() );
	squareSize     = Number(prefsFile.readln() );
	prefsFile.close();
}
	
/* 
 * Display alert and get out of Dodge
 */
function goodBye(msg){
  alert(msg);
  exit();
}

/*
 * Add a color to the document
 */
function addColor(cy, mg, yl, bl){
	var newColor  = theDoc.colors.add();
	var colorName = "C=" + cy + " M=" + mg + " Y=" + yl + " K=" + bl;
	try{
		with(newColor){
			space = ColorSpace.cmyk;
			colorValue = [cy, mg, yl, bl];
			name = colorName;
		}
		with(theDoc.pages[page]){
			with(rectangles.add()){
				strokeWeight  = 0;
				contentType   = ContentType.unassigned;
				fillColor     = newColor;
				fillTint      = 100;
				visibleBounds = [boxOffset + ((yPosition)*boxHeightAdjust),
				                 boxOffset + ((xPosition)*boxWidthAdjust), 
				                 boxOffset + ((yPosition)*boxHeightAdjust) + squareSize,
				                 boxOffset + ((xPosition)*boxWidthAdjust)  + squareSize];
			}	
			with(textFrames.add()){
				contentType   = ContentType.textType;
				contents      = colorName;
				visibleBounds = [boxOffset + (yPosition)*boxHeightAdjust + squareSize + 0.08*boxOffset, 
				                 boxOffset + (xPosition)*boxWidthAdjust  - 0.375*boxOffset,
				                 boxOffset + (yPosition)*boxHeightAdjust + squareSize + 0.9152*boxOffset,
				                 boxOffset + (xPosition)*boxWidthAdjust  + squareSize + 0.375*boxOffset];
			}
		}
		if(xPosition<boxesX)
			xPosition++;
		else{
			xPosition = 0;
			yPosition++;
			if(yPosition > boxesY){
				page++;
				yPosition = 0;
			}
		}
	}
	catch(e){ // Color was already in document, delete the new one.
		newColor.remove();
	}
}